# -*- coding: utf-8 -*-
from .base import Base, Response


class Collection(Base):
    async def details(self, collection_id: int) -> Response:
        """Get collection details by id.

        See more: https://developers.themoviedb.org/3/collections/get-collection-details
        """
        return await self.request(f"collection/{collection_id}")

    async def images(self, collection_id: int) -> Response:
        """Get the images for a collection by id.

        See more: https://developers.themoviedb.org/3/collections/get-collection-images
        """
        return await self.request(f"collection/{collection_id}/images")

    async def translations(self, collection_id: int) -> Response:
        """Get the list translations for a collection by id.

        See more: https://developers.themoviedb.org/3/collections/get-collection-translations
        """
        return await self.request(f"collection/{collection_id}/translations")

    async def search(self, query: str, *, page: int = 1) -> Response:
        """Search for collections.

        See more: https://developers.themoviedb.org/3/search/search-collections
        """
        return await self.request("search/collection", query=query, page=page)



# -*- coding: utf-8 -*-
from .base import Base, Response


class Movie(Base):
    async def details(self, movie_id: int, *, append: str = None, image_language: str = "null") -> Response:
        """Get the primary information about a movie.

        See more: https://developers.themoviedb.org/3/movies/get-movie-details
        """
        return await self.request(f"movie/{movie_id}", append_to_response=append, include_image_language=image_language)

    async def alternative_titles(self, movie_id: int, *, country: str = None) -> Response:
        """Get all of the alternative titles for a movie.

        See more: https://developers.themoviedb.org/3/movies/get-movie-alternative-titles
        """
        return await self.request(f"movie/{movie_id}/alternative_titles", country=country)

    async def certifications(self) -> Response:
        """Get an up to date list of the officially supported movie certifications on TMDB.

        See more: https://developers.themoviedb.org/3/certifications/get-movie-certifications
        """
        return await self.request("certification/movie/list")

    async def changes(self, movie_id: int, *, start_date: str = None, end_date: str = None, page: int = 1) -> Response:
        """Get the changes for a movie.

        See more: https://developers.themoviedb.org/3/movies/get-movie-changes
        """
        return await self.request(f"movie/{movie_id}/changes", start_date=start_date, end_date=end_date, page=page)

    async def credits(self, movie_id: int) -> Response:
        """Get the cast and crew for a movie.

        See more: https://developers.themoviedb.org/3/movies/get-movie-credits
        """
        return await self.request(f"movie/{movie_id}/credits")

    async def discover(
        self,
        *,
        page: int = 1,
        sort_by: str = "popularity.desc",
        certification_country: str = None,
        certification: str = None,
        certification__lte: str = None,
        certification__gte: str = None,
        include_adult: bool = None,
        include_video: bool = None,
        primary_release_year: int = None,
        primary_release_date__gte: str = None,
        primary_release_date__lte: str = None,
        release_date__gte: str = None,
        release_date__lte: str = None,
        with_release_type: int = None,
        year: int = None,
        vote_count__gte: int = None,
        vote_count__lte: int = None,
        vote_average__gte: float = None,
        vote_average__lte: float = None,
        with_cast: str = None,
        with_crew: str = None,
        with_people: str = None,
        with_companies: str = None,
        with_genres: str = None,
        without_genres: str = None,
        with_keywords: str = None,
        without_keywords: str = None,
        with_runtime__gte: int = None,
        with_runtime__lte: int = None,
        with_original_language: str = None,
        with_watch_providers: str = None,
        watch_region: str = None,
        with_watch_monetization_types: str = None,
    ) -> Response:
        """Discover movies by different types of data.

        See more: https://developers.themoviedb.org/3/discover/movie-discover
        """
        return await self.request(
            "discover/movie",
            page=page,
            sort_by=sort_by,
            watch_region=watch_region,
            certification_country=certification_country,
            certification=certification,
            certification__lte=certification__lte,
            certification__gte=certification__gte,
            include_adult=include_adult,
            include_video=include_video,
            primary_release_year=primary_release_year,
            primary_release_date__gte=primary_release_date__gte,
            primary_release_date__lte=primary_release_date__lte,
            release_date__gte=release_date__gte,
            release_date__lte=release_date__lte,
            with_release_type=with_release_type,
            year=year,
            vote_count__gte=vote_count__gte,
            vote_count__lte=vote_count__lte,
            vote_average__gte=vote_average__gte,
            vote_average__lte=vote_average__lte,
            with_cast=with_cast,
            with_crew=with_crew,
            with_people=with_people,
            with_companies=with_companies,
            with_genres=with_genres,
            without_genres=without_genres,
            with_keywords=with_keywords,
            without_keywords=without_keywords,
            with_runtime__gte=with_runtime__gte,
            with_runtime__lte=with_runtime__lte,
            with_original_language=with_original_language,
            with_watch_providers=with_watch_providers,
            with_watch_monetization_types=with_watch_monetization_types,
        )

    async def external_ids(self, movie_id: int) -> Response:
        """Get the external ids for a movie.

        See more: https://developers.themoviedb.org/3/movies/get-movie-external-ids
        """
        return await self.request(f"movie/{movie_id}/external_ids")

    async def genres(self) -> Response:
        """Get the list of official genres for movies.

        See more: https://developers.themoviedb.org/3/genres/get-movie-list
        """
        return await self.request("genre/movie/list")

    async def keywords(self, movie_id: int) -> Response:
        """Get the keywords that have been added to a movie.

        See more: https://developers.themoviedb.org/3/movies/get-movie-keywords
        """
        return await self.request(f"movie/{movie_id}/keywords")

    async def images(self, movie_id: int, *, include_image_language: str = None) -> Response:
        """Get the images that belong to a movie.

        See more: https://developers.themoviedb.org/3/movies/get-movie-images
        """
        return await self.request(f"movie/{movie_id}/images", include_image_language=include_image_language)

    async def latest(self) -> Response:
        """Get the most newly created movie.

        See more: https://developers.themoviedb.org/3/movies/get-latest-movie
        """
        return await self.request("movie/latest")

    async def lists(self, movie_id: int, *, page: int = 1) -> Response:
        """Get a list of lists that this movie belongs to.

        See more: https://developers.themoviedb.org/3/movies/get-movie-lists
        """
        return await self.request(f"movie/{movie_id}/lists", page=page)

    async def now_playing(self, *, page: int = 1) -> Response:
        """Get a list of movies in theatres.

        See more: https://developers.themoviedb.org/3/movies/get-now-playing
        """
        return await self.request("movie/now_playing", page=page)

    async def popular(self, *, page: int = 1) -> Response:
        """Get a list of the current popular movies on TMDB.

        See more: https://developers.themoviedb.org/3/movies/get-popular-movies
        """
        return await self.request("movie/popular", page=page)

    async def providers(self, movie_id: int) -> Response:
        """Get a list of the availabilities per country by provider.

        See more: https://developers.themoviedb.org/3/movies/get-movie-watch-providers
        """
        return await self.request(f"movie/{movie_id}/watch/providers")

    async def providers_list(self) -> Response:
        """Returns a list of the watch provider (OTT/streaming) data we have available for movies.

        See more: https://developers.themoviedb.org/3/watch-providers/get-movie-providers
        """
        return await self.request("watch/providers/movie", watch_region=self.region)

    async def recommendations(self, movie_id: int, *, page: int = 1) -> Response:
        """Get a list of recommended movies for a movie.

        See more: https://developers.themoviedb.org/3/movies/get-movie-recommendations
        """
        return await self.request(f"movie/{movie_id}/recommendations", page=page)

    async def release_dates(self, movie_id: int, *, page: int = 1) -> Response:
        """Get the release date along with the certification for a movie.

        See more: https://developers.themoviedb.org/3/movies/get-movie-release-dates
        """
        return await self.request(f"movie/{movie_id}/release_dates", page=page)

    async def reviews(self, movie_id: int, *, page: int = 1) -> Response:
        """Get the user reviews for a movie.

        See more: https://developers.themoviedb.org/3/movies/get-movie-reviews
        """
        return await self.request(f"movie/{movie_id}/reviews", page=page)

    async def search(
        self,
        query: str,
        *,
        page: int = 1,
        include_adult: bool = False,
        year: int = None,
        primary_release_year: int = None,
    ) -> Response:
        """Search for movies.

        See more: https://developers.themoviedb.org/3/search/search-movies
        """
        return await self.request(
            "search/movie",
            query=query,
            page=page,
            include_adult=include_adult,
            year=year,
            primary_release_year=primary_release_year,
        )

    async def similar(self, movie_id: int, *, page: int = 1) -> Response:
        """Get a list of similar movies.

        See more: https://developers.themoviedb.org/3/movies/get-similar-movies
        """
        return await self.request(f"movie/{movie_id}/similar", page=page)

    async def top_rated(self, *, page: int = 1) -> Response:
        """Get the top rated movies on TMDB.

        See more: https://developers.themoviedb.org/3/movies/get-top-rated-movies
        """
        return await self.request("movie/top_rated", page=page)

    async def translations(self, movie_id: int) -> Response:
        """Get a list of translations that have been created for a movie.

        See more: https://developers.themoviedb.org/3/movies/get-movie-translations
        """
        return await self.request(f"movie/{movie_id}/translations")

    async def trending(self, *, interval: str = "day") -> Response:
        """Get the daily (`interval=day`) or weekly (`interval=week`) trending movies.

        See more: https://developers.themoviedb.org/3/trending/get-trending
        """
        return await self.request(f"trending/movie/{interval}")

    async def upcoming(self, *, page: int = 1) -> Response:
        """Get a list of upcoming movies in theatres.

        See more: https://developers.themoviedb.org/3/movies/get-upcoming
        """
        return await self.request("movie/upcoming", page=page)

    async def videos(self, movie_id: int, *, page: int = 1) -> Response:
        """Get the videos that have been added to a movie.

        See more: https://developers.themoviedb.org/3/movies/get-movie-videos
        """
        return await self.request(f"movie/{movie_id}/videos", page=page)



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from typing import Optional


@dataclass
class Country:
    iso_3166_1: Optional[str]
    name: Optional[str]

    def __str__(self) -> str:
        return self.name



# -*- coding: utf-8 -*-
from .base import Base, Response


class Show(Base):
    async def details(self, tv_id: int, *, append: str = None, image_language: str = "null") -> Response:
        """Get the primary TV show details by id.

        See more: https://developers.themoviedb.org/3/tv/get-tv-details
        """
        return await self.request(f"tv/{tv_id}", append_to_response=append, include_image_language=image_language)

    async def aggregate_credits(self, tv_id: int) -> Response:
        """Get the aggregate credits (cast and crew) that have been added to a TV show.

        See more: https://developers.themoviedb.org/3/tv/get-tv-aggregate-credits
        """
        return await self.request(f"tv/{tv_id}/aggregate_credits")

    async def airing_today(self, *, page: int = 1) -> Response:
        """Get a list of TV shows that are airing today.

        See more: https://developers.themoviedb.org/3/tv/get-tv-airing-today
        """
        return await self.request("tv/airing_today", page=page)

    async def alternative_titles(self, tv_id: int) -> Response:
        """Get all of the alternative titles for a TV show.

        See more: https://developers.themoviedb.org/3/tv/get-tv-alternative-titles
        """
        return await self.request(f"tv/{tv_id}/alternative_titles")

    async def certifications(self) -> Response:
        """Get an up to date list of the officially supported TV show certifications on TMDB.

        See more: https://developers.themoviedb.org/3/certifications/get-tv-certifications
        """
        return await self.request("certification/tv/list")

    async def changes(self, tv_id: int, *, start_date: str = None, end_date: str = None, page: int = 1) -> Response:
        """Get the changes for a TV show.

        See more: https://developers.themoviedb.org/3/tv/get-tv-changes
        """
        return await self.request(f"tv/{tv_id}/changes", start_date=start_date, end_date=end_date, page=page)

    async def content_ratings(self, tv_id: int) -> Response:
        """Get the list of content ratings (certifications) that have been added to a TV show.

        See more: https://developers.themoviedb.org/3/tv/get-tv-content-ratings
        """
        return await self.request(f"tv/{tv_id}/content_ratings")

    async def credits(self, tv_id: int) -> Response:
        """Get the credits (cast and crew) that have been added to a TV show.

        See more: https://developers.themoviedb.org/3/tv/get-tv-credits
        """
        return await self.request(f"tv/{tv_id}/credits")

    async def discover(
        self,
        *,
        page: int = 1,
        sort_by: str = "popularity.desc",
        air_date__gte: str = None,
        air_date__lte: str = None,
        first_air_date__gte: str = None,
        first_air_date__lte: str = None,
        first_air_date_year: int = None,
        timezone: str = None,
        vote_average__gte: float = None,
        vote_count__gte: int = None,
        with_genres: str = None,
        with_networks: str = None,
        without_genres: str = None,
        with_runtime__gte: int = None,
        with_runtime__lte: int = None,
        include_null_first_air_dates: bool = None,
        with_original_language: str = None,
        without_keywords: str = None,
        screened_theatrically: bool = None,
        with_companies: str = None,
        with_keywords: str = None,
        with_watch_providers: str = None,
        watch_region: str = None,
        with_watch_monetization_types: str = None,
    ) -> Response:
        """Discover TV shows by different types of data.

        See more: https://developers.themoviedb.org/3/discover/tv-discover
        """
        return await self.request(
            "discover/tv",
            page=page,
            sort_by=sort_by,
            watch_region=watch_region,
            air_date__gte=air_date__gte,
            air_date__lte=air_date__lte,
            first_air_date__gte=first_air_date__gte,
            first_air_date__lte=first_air_date__lte,
            first_air_date_year=first_air_date_year,
            timezone=timezone,
            vote_average__gte=vote_average__gte,
            vote_count__gte=vote_count__gte,
            with_genres=with_genres,
            with_networks=with_networks,
            without_genres=without_genres,
            with_runtime__gte=with_runtime__gte,
            with_runtime__lte=with_runtime__lte,
            include_null_first_air_dates=include_null_first_air_dates,
            with_original_language=with_original_language,
            without_keywords=without_keywords,
            screened_theatrically=screened_theatrically,
            with_companies=with_companies,
            with_keywords=with_keywords,
            with_watch_providers=with_watch_providers,
            with_watch_monetization_types=with_watch_monetization_types,
        )

    async def episode_groups(self, tv_id: int) -> Response:
        """Get all of the episode groups that have been created for a TV show.

        See more: https://developers.themoviedb.org/3/tv/get-tv-episode-groups
        """
        return await self.request(f"tv/{tv_id}/episode_groups")

    async def external_ids(self, tv_id: int) -> Response:
        """Get the external ids for a TV show.

        See more: https://developers.themoviedb.org/3/tv/get-tv-external-ids
        """
        return await self.request(f"tv/{tv_id}/external_ids")

    async def genres(self) -> Response:
        """Get the list of official genres for TV shows.

        See more: https://developers.themoviedb.org/3/genres/get-tv-list
        """
        return await self.request("genre/tv/list")

    async def images(self, tv_id: int) -> Response:
        """Get the images that belong to a TV show.

        See more: https://developers.themoviedb.org/3/tv/get-tv-images
        """
        return await self.request(f"tv/{tv_id}/images")

    async def keywords(self, tv_id: int) -> Response:
        """Get the keywords that have been added to a TV show.

        See more: https://developers.themoviedb.org/3/tv/get-tv-keywords
        """
        return await self.request(f"tv/{tv_id}/keywords")

    async def latest(self) -> Response:
        """Get the most newly created TV show.

        See more: https://developers.themoviedb.org/3/tv/get-latest-tv
        """
        return await self.request("tv/latest")

    async def on_the_air(self, *, page: int = 1) -> Response:
        """Get a list of TV shows that are airing today.

        See more: https://developers.themoviedb.org/3/tv/get-tv-on-the-air
        """
        return await self.request("tv/on_the_air", page=page)

    async def popular(self, *, page: int = 1) -> Response:
        """Get a list of the current popular TV shows on TMDB.

        See more: https://developers.themoviedb.org/3/tv/get-popular-tv-shows
        """
        return await self.request("tv/popular", page=page)

    async def providers(self, tv_id: int) -> Response:
        """Get a list of the availabilities per country by provider.

        See more: https://developers.themoviedb.org/3/tv/get-tv-watch-providers
        """
        return await self.request(f"tv/{tv_id}/watch/providers")

    async def providers_list(self, *, watch_region=None) -> Response:
        """Returns a list of the watch provider (OTT/streaming) data we have available for TV series.

        See more: https://developers.themoviedb.org/3/watch-providers/get-tv-providers
        """
        return await self.request("watch/providers/tv", watch_region=watch_region)

    async def recommendations(self, tv_id: int, *, page: int = 1) -> Response:
        """Get the list of TV show recommendations for this item.

        See more: https://developers.themoviedb.org/3/tv/get-tv-recommendations
        """
        return await self.request(f"tv/{tv_id}/recommendations", page=page)

    async def reviews(self, tv_id: int, *, page: int = 1) -> Response:
        """Get the reviews for a TV show.

        See more: https://developers.themoviedb.org/3/tv/get-tv-reviews
        """
        return await self.request(f"tv/{tv_id}/reviews", page=page)

    async def screened_theatrically(self, tv_id: int) -> Response:
        """Get a list of seasons or episodes that have been screened in a film festival or theatre.

        See more: https://developers.themoviedb.org/3/tv/get-screened-theatrically
        """
        return await self.request(f"tv/{tv_id}/screened_theatrically")

    async def search(
        self,
        query: str,
        *,
        page: int = 1,
        include_adult: bool = False,
        first_air_date_year: int = None,
    ) -> Response:
        """Search for a TV show.

        See more: https://developers.themoviedb.org/3/search/search-tv-shows
        """
        return await self.request(
            "search/tv",
            query=query,
            page=page,
            include_adult=include_adult,
            first_air_date_year=first_air_date_year,
        )

    async def similar(self, tv_id: int, *, page: int = 1) -> Response:
        """Get a list of similar TV shows.

        See more: https://developers.themoviedb.org/3/tv/get-similar-tv-shows
        """
        return await self.request(f"tv/{tv_id}/similar", page=page)

    async def top_rated(self, *, page: int = 1) -> Response:
        """Get a list of the top rated TV shows on TMDB.

        See more: https://developers.themoviedb.org/3/tv/get-top-rated-tv
        """
        return await self.request("tv/top_rated", page=page)

    async def translations(self, tv_id: int) -> Response:
        """Get a list of the translations that exist for a TV show.

        See more: https://developers.themoviedb.org/3/tv/get-tv-translations
        """
        return await self.request(f"tv/{tv_id}/translations")

    async def trending(self, *, interval: str = "day") -> Response:
        """Get the daily (`interval=day`) or weekly (`interval=week`) trending TV shows.

        See more: https://developers.themoviedb.org/3/trending/get-trending
        """
        return await self.request(f"trending/tv/{interval}")

    async def videos(self, tv_id: int) -> Response:
        """Get the videos that have been added to a TV show.

        See more: https://developers.themoviedb.org/3/tv/get-tv-videos
        """
        return await self.request(f"tv/{tv_id}/videos")



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from datetime import date
from typing import Optional

from .external_id import ExternalIDs
from .image import Images
from .media import Media


# @dataclass
# class Person:
#     id: Optional[int]
#     gender: Optional[int]
#     name: Optional[str]
#     known_for_department: Optional[str]
#     profile_path: Optional[str]
#     known_for: Optional[list[Media]]
#     adult: Optional[bool]
#     popularity: Optional[float]

#     @property
#     def profile(self) -> Optional[str]:
#         if self.profile_path:
#             return f"https://image.tmdb.org/t/p/w1280{self.profile_path}"


# @dataclass
# class Job:
#     id: Optional[str]
#     credit_type: Optional[str]
#     department: Optional[str]
#     job: Optional[str]
#     media: Optional[Media]
#     media_type: Optional[str]
#     person: Optional[Person]


@dataclass
class PersonPartial:
    id: Optional[int]
    adult: Optional[bool]
    gender: Optional[int]
    known_for: Optional[list[Media]]
    known_for_department: Optional[str]
    name: Optional[str]
    popularity: Optional[float]
    profile_path: Optional[str]

    def __str__(self) -> str:
        return self.name

    @property
    def profile(self) -> Optional[str]:
        if self.profile_path:
            return f"https://image.tmdb.org/t/p/w1280{self.profile_path}"


@dataclass
class Job:
    id: Optional[int]
    adult: Optional[bool]
    backdrop_path: Optional[str]
    character: Optional[str]
    credit_id: Optional[str]
    department: Optional[str]
    episode_count: Optional[int]
    first_air_date: Optional[date]
    genre_ids: Optional[list[int]]
    job: Optional[str]
    media_type: Optional[str]
    name: Optional[str]
    order: Optional[int]
    origin_country: Optional[list[str]]
    original_language: Optional[str]
    original_name: Optional[str]
    original_title: Optional[str]
    overview: Optional[str]
    popularity: Optional[float]
    poster_path: Optional[str]
    release_date: Optional[date]
    title: Optional[str]
    video: Optional[bool]
    vote_average: Optional[float]
    vote_count: Optional[int]

    def __post_init__(self) -> None:
        self.name = self.title or self.name or self.original_title or self.original_name

    @property
    def description(self) -> Optional[str]:
        return self.overview

    @property
    def date(self) -> Optional[date]:
        return self.release_date or self.first_air_date

    @property
    def year(self) -> Optional[str]:
        if self.date:
            return str(self.date.year)

    @property
    def rating(self) -> Optional[str]:
        if self.vote_average and self.vote_count:
            return f"{self.vote_average} ({self.vote_count})"
        if self.vote_average:
            return str(self.vote_average)

    @property
    def backdrop(self) -> Optional[str]:
        if self.backdrop_path:
            return f"https://image.tmdb.org/t/p/w1280{self.backdrop_path}"

    @property
    def poster(self) -> Optional[str]:
        if self.poster_path:
            return f"https://image.tmdb.org/t/p/w1280{self.poster_path}"


@dataclass
class Jobs:
    id: Optional[int]
    cast: Optional[list[Job]]
    crew: Optional[list[Job]]

    def __bool__(self) -> bool:
        return self.cast or self.crew


@dataclass
class People:
    page: Optional[int]
    results: Optional[list[PersonPartial]]
    total_pages: Optional[int]
    total_results: Optional[int]

    def __bool__(self) -> bool:
        return self.total_results > 0

    def __iter__(self) -> iter:
        return iter(self.results)

    def __getitem__(self, index: int) -> PersonPartial:
        return self.results[index]


@dataclass
class Person:
    id: Optional[int]
    adult: Optional[bool]
    also_known_as: Optional[list[str]]
    biography: Optional[str]
    birthday: Optional[date]
    deathday: Optional[date]
    gender: Optional[int]
    homepage: Optional[str]
    imdb_id: Optional[str]
    known_for_department: Optional[str]
    name: Optional[str]
    place_of_birth: Optional[str]
    popularity: Optional[float]
    profile_path: Optional[str]
    combined_credits: Optional[Jobs]
    external_ids: Optional[ExternalIDs]
    images: Optional[Images]
    movie_credits: Optional[Jobs]
    tv_credits: Optional[Jobs]

    def __str__(self) -> str:
        return self.name

    @property
    def description(self) -> Optional[str]:
        return self.biography

    @property
    def date(self) -> Optional[date]:
        return self.birthday

    @property
    def year(self) -> Optional[str]:
        if self.date:
            return str(self.date.year)

    @property
    def age(self) -> Optional[str]:
        age = date.today() - self.birthday
        return age.year

    @property
    def known_for(self) -> Optional[list]:
        if self.known_for_department == "Acting":
            return self.combined_credits.cast
        return self.combined_credits.crew

    @property
    def profile(self) -> Optional[str]:
        if self.backdrop_path:
            return f"https://image.tmdb.org/t/p/w1280{self.profile_path}"

    @property
    def imdb(self) -> Optional[str]:
        if self.external_ids:
            return self.external_ids.imdb("person")



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from typing import Optional


@dataclass
class AlternativeName:
    name: Optional[str]
    type: Optional[str]

    def __str__(self) -> str:
        return self.name


@dataclass
class AlternativeNames:
    id: Optional[int]
    alternative_names: Optional[list[AlternativeName]]
    results: Optional[list[AlternativeName]]

    def __bool__(self) -> bool:
        return self.alternative_names or self.results

    def __iter__(self) -> iter:
        return iter(self.alternative_names or self.results)

    def __getitem__(self, index: int) -> AlternativeName:
        return (self.alternative_names or self.results)[index]



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from typing import Optional


@dataclass
class Credit:
    id: Optional[int]
    adult: Optional[bool]
    cast_id: Optional[int]
    character: Optional[str]
    credit_id: Optional[str]
    department: Optional[str]
    gender: Optional[int]
    job: Optional[str]
    known_for_department: Optional[str]
    name: Optional[str]
    order: Optional[int]
    original_name: Optional[str]
    popularity: Optional[float]
    profile_path: Optional[str]

    def __str__(self) -> str:
        return self.name or self.original_name

    @property
    def profile(self) -> Optional[str]:
        if self.profile_path:
            return f"https://image.tmdb.org/t/p/w1280{self.profile_path}"

    def is_director(self) -> bool:
        return self.job == "Director"


@dataclass
class Credits:
    id: Optional[int]
    cast: Optional[list[Credit]]
    crew: Optional[list[Credit]]

    def __bool__(self) -> bool:
        return self.cast or self.crew

    @property
    def directors(self) -> Optional[list[str]]:
        if self.crew:
            return [str(person) for person in self.crew if person.is_director()]



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from typing import Optional


@dataclass
class Image:
    aspect_ratio: Optional[float]
    file_path: Optional[str]
    height: Optional[int]
    iso_639_1: Optional[str]
    vote_average: Optional[float]
    vote_count: Optional[int]
    width: Optional[int]

    def __str__(self) -> str:
        return self.file

    @property
    def file(self) -> Optional[str]:
        if self.file_path:
            return f"https://image.tmdb.org/t/p/w1280{self.file_path}"

    @property
    def rating(self) -> Optional[str]:
        if self.vote_average and self.vote_count:
            return f"{self.vote_average} ({self.vote_count})"
        if self.vote_average:
            return str(self.vote_average)


@dataclass
class Images:
    id: Optional[int]
    backdrops: Optional[list[Image]]
    logos: Optional[list[Image]]
    posters: Optional[list[Image]]
    profiles: Optional[list[Image]]

    def __bool__(self) -> bool:
        return (
            self.backdrops
            or self.logos
            or self.posters
            or self.tv_results
            or self.profiles
        )



# -*- coding: utf-8 -*-
from .alternative_name import AlternativeName, AlternativeNames  # NOQA
from .collection import Collection, Collections  # NOQA
from .company import Companies, Company  # NOQA
from .country import Country  # NOQA
from .credit import Credit, Credits  # NOQA
from .external_id import ExternalIDs  # NOQA
from .find import Find  # NOQA
from .genre import Genre  # NOQA
from .image import Image, Images  # NOQA
from .keyword import Keyword, Keywords  # NOQA
from .language import Language  # NOQA
from .media import Media  # NOQA
from .movie import Movie, Movies  # NOQA
from .person import People, Person, PersonPartial  # NOQA
from .provider import Provider, Providers  # NOQA
from .title import Title, Titles  # NOQA
from .translaction import Translation  # NOQA
from .tv_episode import Episode  # NOQA
from .tv_season import Season  # NOQA
from .tv_show import Show, Shows  # NOQA
from .video import Video, Videos  # NOQA



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from datetime import date
from typing import Optional


@dataclass
class Video:
    id: Optional[str]
    iso_3166_1: Optional[str]
    iso_639_1: Optional[str]
    key: Optional[str]
    name: Optional[str]
    official: Optional[bool]
    published_at: Optional[date]
    site: Optional[str]
    size: Optional[int]
    type: Optional[str]

    def __str__(self) -> str:
        return self.name

    @property
    def date(self) -> Optional[date]:
        return self.published_at

    @property
    def year(self) -> Optional[str]:
        if self.date:
            return str(self.date.year)

    @property
    def url(self) -> Optional[str]:
        if self.site == "Youtube":
            return f"https://youtube.com/watch?v={self.key}"


@dataclass
class Videos:
    id: Optional[int]
    results: Optional[list[Video]]

    def __bool__(self) -> bool:
        return self.results

    def __iter__(self) -> iter:
        return iter(self.results)

    def __getitem__(self, index: int) -> Video:
        return self.results[index]



# -*- coding: utf-8 -*-
from .base import Base, Response


class Season(Base):
    async def details(
        self,
        tv_id: int,
        season_number: int,
        *,
        append_to_response: str = None,
    ) -> Response:
        """Get the TV season details by id.

        See more: https://developers.themoviedb.org/3/tv-seasons/get-tv-season-details
        """
        return await self.request(f"tv/{tv_id}/season/{season_number}", append_to_response=append_to_response)

    async def aggregate_credits(self, tv_id: int, season_number: int) -> Response:
        """Get the aggregate credits for TV season.

        See more: https://developers.themoviedb.org/3/tv-seasons/get-tv-season-aggregate-credits
        """
        return await self.request(f"tv/{tv_id}/season/{season_number}/aggregate_credits")

    async def alternative_titles(self, tv_id: int, season_number: int) -> Response:
        """Get all of the alternative titles for a TV show.

        See more: https://developers.themoviedb.org/3/tv/get-tv-alternative-titles
        """
        return await self.request(f"tv/{tv_id}/season/{season_number}/alternative_titles")

    async def changes(self, tv_id: int, season_number: int, *, start_date: str = None, end_date: str = None, page: int = 1) -> Response:
        """Get the changes for a TV season.

        See more: https://developers.themoviedb.org/3/tv-seasons/get-tv-season-changes
        """
        return await self.request(f"tv/{tv_id}/season/{season_number}/changes", start_date=start_date, end_date=end_date, page=page)

    async def credits(self, tv_id: int, season_number: int) -> Response:
        """Get the credits for TV season.

        See more: https://developers.themoviedb.org/3/tv-seasons/get-tv-season-credits
        """
        return await self.request(f"tv/{tv_id}/season/{season_number}/credits")

    async def external_ids(self, tv_id: int, season_number: int) -> Response:
        """Get the external ids for a TV season.

        See more: https://developers.themoviedb.org/3/tv-seasons/get-tv-season-external-ids
        """
        return await self.request(f"tv/{tv_id}/season/{season_number}/external_ids")

    async def images(self, tv_id: int, season_number: int) -> Response:
        """Get the images that belong to a TV season.

        See more: https://developers.themoviedb.org/3/tv-seasons/get-tv-season-images
        """
        return await self.request(f"tv/{tv_id}/season/{season_number}/images")

    async def translations(self, tv_id: int, season_number: int) -> Response:
        """Get the credits for TV season.

        See more: https://developers.themoviedb.org/3/tv-seasons/get-tv-season-translations
        """
        return await self.request(f"tv/{tv_id}/season/{season_number}/translations")

    async def videos(self, tv_id: int, season_number: int) -> Response:
        """Get the videos that have been added to a TV season.

        See more: https://developers.themoviedb.org/3/tv-seasons/get-tv-season-videos
        """
        return await self.request(f"tv/{tv_id}/season/{season_number}/videos")



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from typing import Optional

from .media import Media
from .person import PersonPartial


@dataclass
class Find:
    movie_results: Optional[list[Media]]
    person_results: Optional[list[PersonPartial]]
    tv_episode_results: Optional[list[Media]]
    tv_results: Optional[list[Media]]
    tv_season_results: Optional[list[Media]]

    def __bool__(self) -> bool:
        return (
            self.movie_results
            or self.person_results
            or self.tv_episode_results
            or self.tv_results
            or self.tv_season_results
        )



# -*- coding: utf-8 -*-
from .base import Base, Response


class Company(Base):
    async def details(self, company_id: int) -> Response:
        """Get a companies details by id.

        See more: https://developers.themoviedb.org/3/companies/get-company-details
        """
        return await self.request(f"company/{company_id}")

    async def alternative_names(self, company_id: int) -> Response:
        """Get the alternative names of a company.

        See more: https://developers.themoviedb.org/3/companies/get-company-alternative-names
        """
        return await self.request(f"company/{company_id}/alternative_names")

    async def images(self, company_id: int) -> Response:
        """Get a companies logos by id.

        See more: https://developers.themoviedb.org/3/companies/get-company-images
        """
        return await self.request(f"company/{company_id}/images")

    async def search(self, query: str, *, page: int = 1) -> Response:
        """Search for companies.

        See more: https://developers.themoviedb.org/3/search/search-companies
        """
        return await self.request("search/company", query=query, page=page)



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from typing import Optional


@dataclass
class Flatrate:
    display_priority: Optional[int]
    logo_path: Optional[str]
    provider_id: Optional[int]
    provider_name: Optional[str]

    def __str__(self) -> str:
        return self.provider_name

    @property
    def logo(self) -> Optional[str]:
        if self.logo_path:
            return f"https://image.tmdb.org/t/p/w1280{self.logo_path}"


@dataclass
class ProviderData:
    buy: Optional[list[Flatrate]]
    flatrate: Optional[list[Flatrate]]
    free: Optional[list[Flatrate]]
    link: Optional[str]

    def __str__(self) -> str:
        return self.link

    def __bool__(self) -> bool:
        return self.buy or self.flatrate or self.free


@dataclass
class Provider:
    BR: Optional[ProviderData]
    US: Optional[ProviderData]


@dataclass
class Providers:
    id: Optional[int]
    results: Optional[Provider]

    def __bool__(self) -> bool:
        return self.results

    def __getitem__(self, region: str) -> ProviderData:
        return self.results[region.upper()]



# -*- coding: utf-8 -*-
from .base import Base, Response


class Find(Base):
    async def by_imdb(self, imdb_id: str) -> Response:
        """Search movies, TV shows and people by IMDb ID.

        See more: https://developers.themoviedb.org/3/find/find-by-id
        """
        return await self.request(f"find/{imdb_id}", external_source="imdb_id")

    async def by_tvdb(self, tvdb_id: str) -> Response:
        """Search movies, TV shows and people by TVDb ID.

        See more: https://developers.themoviedb.org/3/find/find-by-id
        """
        return await self.request(f"find/{tvdb_id}", external_source="tvdb_id")

    async def by_freebase_mid(self, freebase_mid: str) -> Response:
        """Search movies, TV shows and people by Freebase MID.

        See more: https://developers.themoviedb.org/3/find/find-by-id
        """
        return await self.request(f"find/{freebase_mid}", external_source="freebase_mid")

    async def by_freebase(self, freebase_id: str) -> Response:
        """Search movies, TV shows and people by Freebase ID.

        See more: https://developers.themoviedb.org/3/find/find-by-id
        """
        return await self.request(f"find/{freebase_id}", external_source="freebase_id")

    async def by_tvrage(self, tvrage_id: str) -> Response:
        """Search movies, TV shows and people by TVRage ID.

        See more: https://developers.themoviedb.org/3/find/find-by-id
        """
        return await self.request(f"find/{tvrage_id}", external_source="tvrage_id")

    async def by_facebook(self, facebook_id: str) -> Response:
        """Search movies, TV shows and people by Facebook ID.

        See more: https://developers.themoviedb.org/3/find/find-by-id
        """
        return await self.request(f"find/{facebook_id}", external_source="facebook_id")

    async def by_instagram(self, instagram_id: str) -> Response:
        """Search movies, TV shows and people by Instagram ID.

        See more: https://developers.themoviedb.org/3/find/find-by-id
        """
        return await self.request(f"find/{instagram_id}", external_source="instagram_id")

    async def by_twitter(self, twitter_id: str) -> Response:
        """Search movies, TV shows and people by Twitter ID.

        See more: https://developers.themoviedb.org/3/find/find-by-id
        """
        return await self.request(f"find/{twitter_id}", external_source="twitter_id")



# -*- coding: utf-8 -*-
import os
from datetime import date
from typing import Any

import aiohttp
from dacite import Config, from_dict


class Response(dict):
    def to(self, class_type: Any) -> Any:
        return from_dict(data_class=class_type, data=self, config=Config(type_hooks={date: date.fromisoformat}))


class Base:
    TMDB_URL = "https://api.themoviedb.org"
    TMDB_VERSION = "3"
    TMDB_KEY = "TMDB_KEY"
    TMDB_LANGUAGE = "TMDB_LANGUAGE"
    TMDB_REGION = "TMDB_REGION"

    def __init__(
        self,
        *,
        session: aiohttp.ClientSession = None,
        language: str = None,
        region: str = None,
    ):
        self._session = session
        self._language = language if language is not None else os.environ.get(self.TMDB_LANGUAGE, "en-US")
        self._region = region if region is not None else os.environ.get(self.TMDB_REGION, "US")

    @property
    def host(self) -> str:
        return self.TMDB_URL

    @property
    def version(self) -> str:
        return self.TMDB_VERSION

    @property
    def key(self) -> str:
        return os.environ.get(self.TMDB_KEY)

    @key.setter
    def key(self, key: str) -> None:
        os.environ[self.TMDB_KEY] = key

    @property
    def language(self) -> str:
        return self._language

    @language.setter
    def language(self, language: str) -> None:
        self._language = language

    @property
    def region(self) -> str:
        return self._region

    @region.setter
    def region(self, region: str) -> None:
        self._region = region

    @property
    def default_params(self) -> dict:
        return {
            "api_key": self.key,
            "language": self.language,
            "region": self.region,
            "watch_region": self.region,
        }

    async def request(self, path: str, method: str = "GET", **kwargs) -> Response:
        session = self._session or aiohttp.ClientSession()
        url = f"{self.host}/{self.version}/{path}"
        params = {
            k.replace("__", "."): "true" if v is True else "false" if v is False else v
            for k, v in kwargs.items()
            if v is not None
        }
        params = {**self.default_params, **params}

        response = await session.request(method, url=url, params=params)
        if self._session is None:
            await session.close()
        if not response.ok:
            response.raise_for_status()
        result = await response.json()

        if "watch/providers" in result:
            result["watch_providers"] = result.pop("watch/providers")
        return Response(result)



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from datetime import date
from typing import Optional

from .company import Company
from .country import Country
from .credit import Credits
from .external_id import ExternalIDs
from .genre import Genre
from .image import Images
from .keyword import Keywords
from .language import Language
from .media import Media
from .provider import Providers
from .title import Titles
from .video import Videos


@dataclass
class Creator:
    id: Optional[int]
    credit_id: Optional[str]
    name: Optional[str]
    gender: Optional[int]
    profile_path: Optional[str]

    def __str__(self) -> str:
        return self.name

    @property
    def profile(self) -> Optional[str]:
        if self.profile_path:
            return f"https://image.tmdb.org/t/p/w1280{self.profile_path}"


@dataclass
class Shows:
    page: Optional[int]
    results: Optional[list[Media]]
    total_pages: Optional[int]
    total_results: Optional[int]

    def __bool__(self) -> bool:
        return self.total_results > 0

    def __iter__(self) -> iter:
        return iter(self.results)

    def __getitem__(self, index: int) -> Media:
        return self.results[index]


@dataclass
class Show:
    id: Optional[int]
    adult: Optional[bool]
    backdrop_path: Optional[str]
    created_by: Optional[list[Creator]]
    episode_run_time: Optional[list[int]]
    first_air_date: Optional[date]
    genres: Optional[list[Genre]]
    homepage: Optional[str]
    in_production: Optional[bool]
    languages: Optional[list[str]]
    last_air_date: Optional[date]
    last_episode_to_air: Optional[Media]
    name: Optional[str]
    networks: Optional[list[Company]]
    next_episode_to_air: Optional[Media]
    number_of_episodes: Optional[int]
    number_of_seasons: Optional[int]
    origin_country: Optional[list[str]]
    original_language: Optional[str]
    original_name: Optional[str]
    overview: Optional[str]
    popularity: Optional[float]
    poster_path: Optional[str]
    production_companies: Optional[list[Company]]
    production_countries: Optional[list[Country]]
    seasons: Optional[list[Media]]
    spoken_languages: Optional[list[Language]]
    status: Optional[str]
    tagline: Optional[str]
    type: Optional[str]
    vote_average: Optional[float]
    vote_count: Optional[int]
    alternative_titles: Optional[Titles]
    credits: Optional[Credits]
    external_ids: Optional[ExternalIDs]
    images: Optional[Images]
    keywords: Optional[Keywords]
    recommendations: Optional[Shows]
    similar: Optional[Shows]
    videos: Optional[Videos]
    watch_providers: Optional[Providers]

    def __post_init__(self) -> None:
        self.name = self.name or self.original_name

    def __str__(self) -> str:
        if self.year:
            return f"{self.name} ({self.year})"
        return self.name

    @property
    def description(self) -> Optional[str]:
        return self.overview

    @property
    def date(self) -> Optional[date]:
        return self.first_air_date

    @property
    def year(self) -> Optional[str]:
        if self.date:
            return str(self.date.year)

    @property
    def rating(self) -> Optional[str]:
        if self.vote_average and self.vote_count:
            return f"{self.vote_average} ({self.vote_count})"
        if self.vote_average:
            return str(self.vote_average)

    @property
    def backdrop(self) -> Optional[str]:
        if self.backdrop_path:
            return f"https://image.tmdb.org/t/p/w1280{self.backdrop_path}"

    @property
    def poster(self) -> Optional[str]:
        if self.poster_path:
            return f"https://image.tmdb.org/t/p/w1280{self.poster_path}"

    @property
    def runtime(self) -> Optional[int]:
        if self.episode_run_time:
            return int(sum(self.episode_run_time) / len(self.episode_run_time))

    @property
    def duration(self) -> Optional[str]:
        if self.runtime:
            hours, minutes = divmod(self.runtime, 60)
            if hours != 0 and minutes != 0:
                return f"{hours}h {minutes}min"
            if hours != 0:
                return f"{hours}h"
            return f"{minutes}min"

    @property
    def duration_total(self) -> Optional[str]:
        if self.runtime and self.number_of_episodes:
            hours, minutes = divmod(self.runtime * self.number_of_episodes, 60)
            if hours != 0 and minutes != 0:
                return f"{hours}h {minutes}min"
            if hours != 0:
                return f"{hours}h"
            return f"{minutes}min"

    @property
    def trailer(self) -> Optional[str]:
        if self.videos and self.videos.results[0].url:
            return self.videos.results[0].url

    @property
    def imdb(self) -> Optional[str]:
        if self.external_ids:
            return self.external_ids.imdb("tv")

    @property
    def directors(self) -> Optional[list[str]]:
        if self.credits:
            return self.credits.directors



# -*- coding: utf-8 -*-
from .base import Base, Response, Response  # NOQA
from .collection import Collection  # NOQA
from .company import Company  # NOQA
from .credit import Credit  # NOQA
from .find import Find  # NOQA
from .keyword import Keyword  # NOQA
from .movie import Movie  # NOQA
from .network import Network  # NOQA
from .person import Person  # NOQA
from .tv_episode import Episode  # NOQA
from .tv_season import Season  # NOQA
from .tv_show import Show  # NOQA



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from typing import Optional


@dataclass
class Title:
    iso_3166_1: Optional[str]
    title: Optional[str]
    type: Optional[str]

    def __str__(self) -> str:
        return self.title


@dataclass
class Titles:
    id: Optional[int]
    results: Optional[list[Title]]
    titles: Optional[list[Title]]

    def __bool__(self) -> bool:
        return self.results or self.titles

    def __iter__(self) -> iter:
        return iter(self.results or self.titles)

    def __getitem__(self, index: int) -> Title:
        return (self.results or self.titles)[index]



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from typing import Optional


@dataclass
class Keyword:
    id: Optional[int]
    name: Optional[str]

    def __str__(self) -> str:
        return self.name


@dataclass
class Keywords:
    id: Optional[int]
    keywords: Optional[list[Keyword]]
    page: Optional[int]
    results: Optional[list[Keyword]]
    total_pages: Optional[int]
    total_results: Optional[int]

    def __bool__(self) -> bool:
        return self.total_results > 0

    def __iter__(self) -> iter:
        return iter(self.results or self.keywords)

    def __getitem__(self, index: int) -> Keyword:
        return (self.keywords or self.results)[index]



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from typing import Optional


@dataclass
class ExternalIDs:
    facebook_id: Optional[str]
    freebase_id: Optional[str]
    freebase_mid: Optional[str]
    imdb_id: Optional[str]
    instagram_id: Optional[str]
    tvdb_id: Optional[int]
    tvrage_id: Optional[str]
    twitter_id: Optional[str]

    @property
    def facebook(self) -> Optional[str]:
        if self.facebook_id:
            return f"https://facebook.com/{self.facebook_id}"

    def freebase(self, media_type: str) -> Optional[str]:
        raise NotImplementedError()

    @property
    def instagram(self) -> Optional[str]:
        if self.instagram_id:
            return f"https://instagram.com/{self.instagram_id}"

    def imdb(self, media_type: str) -> Optional[str]:
        if self.imdb_id and media_type == "person":
            return f"https://www.imdb.com/name/{self.imdb_id}"
        if self.imdb_id and media_type == "movie":
            return f"https://www.imdb.com/title/{self.imdb_id}"
        if self.imdb_id and media_type == "tv":
            return f"https://www.imdb.com/title/{self.imdb_id}"

    def tvdb(self, media_type: str) -> Optional[str]:
        raise NotImplementedError()

    def tvrage(self, media_type: str) -> Optional[str]:
        raise NotImplementedError()

    @property
    def twitter(self) -> Optional[str]:
        if self.twitter_id:
            return f"https://twitter.com/{self.twitter_id}"



# -*- coding: utf-8 -*-
from .base import Base, Response


class Person(Base):
    async def details(self, person_id: int, *, append: str = None, image_language: str = "null") -> Response:
        """Get the primary person details by id.

        See more: https://developers.themoviedb.org/3/people/get-person-details
        """
        return await self.request(f"person/{person_id}", append_to_response=append, include_image_language=image_language)

    async def changes(self, person_id: int, *, start_date: str = None, end_date: str = None, page: int = 1) -> Response:
        """Get the changes for a person.

        See more: https://developers.themoviedb.org/3/people/get-person-changes
        """
        return await self.request(f"person/{person_id}/changes", start_date=start_date, end_date=end_date, page=page)

    async def movie_credits(self, person_id: int) -> Response:
        """Get the movie credits for a person.

        See more: https://developers.themoviedb.org/3/people/get-person-movie-credits
        """
        return await self.request(f"person/{person_id}/movie_credits")

    async def tv_credits(self, person_id: int) -> Response:
        """Get the TV show credits for a person.

        See more: https://developers.themoviedb.org/3/people/get-person-tv-credits
        """
        return await self.request(f"person/{person_id}/tv_credits")

    async def combined_credits(self, person_id: int) -> Response:
        """Get the movie and TV credits together in a single response.

        See more: https://developers.themoviedb.org/3/people/get-person-combined-credits
        """
        return await self.request(f"person/{person_id}/combined_credits")

    async def external_ids(self, person_id: int) -> Response:
        """Get the external ids for a person.

        See more: https://developers.themoviedb.org/3/people/get-person-external-ids
        """
        return await self.request(f"person/{person_id}/external_ids")

    async def images(self, person_id: int) -> Response:
        """Get the images for a person.

        See more: https://developers.themoviedb.org/3/people/get-person-images
        """
        return await self.request(f"person/{person_id}/images")

    async def tagged_images(self, person_id: int, *, page: int = 1) -> Response:
        """Get the images that this person has been tagged in.

        See more: https://developers.themoviedb.org/3/people/get-tagged-images
        """
        return await self.request(f"person/{person_id}/tagged_images", page=page)

    async def keywords(self, person_id: int) -> Response:
        """Get the keywords that have been added to a movie.

        See more: https://developers.themoviedb.org/3/movies/get-movie-keywords
        """
        return await self.request(f"person/{person_id}/keywords")

    async def latest(self) -> Response:
        """Get the most newly created person.

        See more: https://developers.themoviedb.org/3/people/get-latest-person
        """
        return await self.request("person/latest")

    async def popular(self, *, page: int = 1) -> Response:
        """Get the list of popular people on TMDB.

        See more: https://developers.themoviedb.org/3/people/get-popular-people
        """
        return await self.request("person/popular", page=page)

    async def search(
        self, query: str, *, page: int = 1, include_adult: bool = False
    ) -> Response:
        """Search for people.

        See more: https://developers.themoviedb.org/3/search/search-people
        """
        return await self.request("search/person", query=query, page=page, include_adult=include_adult)

    async def translations(self, person_id: int) -> Response:
        """Get a list of translations that have been created for a person.

        See more: https://developers.themoviedb.org/3/people/get-person-translations
        """
        return await self.request(f"person/{person_id}/translations")

    async def trending(self, *, interval: str = "day") -> Response:
        """Get the daily (`interval=day`) or weekly (`interval=week`) trending TV shows.

        See more: https://developers.themoviedb.org/3/trending/get-trending
        """
        return await self.request(f"trending/person/{interval}")



# -*- coding: utf-8 -*-
from .base import Base, Response


class Network(Base):
    async def details(self, network_id: int) -> Response:
        """Get the details of a network.

        See more: https://developers.themoviedb.org/3/networks/get-network-details
        """
        return await self.request(f"network/{network_id}")

    async def alternative_names(self, network_id: int) -> Response:
        """Get the alternative names of a network.

        See more: https://developers.themoviedb.org/3/networks/get-network-alternative-names
        """
        return await self.request(f"network/{network_id}/alternative_names")

    async def images(self, network_id: int) -> Response:
        """Get the TV network logos by id.

        See more: https://developers.themoviedb.org/3/networks/get-network-images
        """
        return await self.request(f"network/{network_id}/images")



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from typing import Optional

from .image import Images
from .media import Media
from .translaction import Translation


@dataclass
class Collection:
    id: Optional[int]
    adult: Optional[bool]
    backdrop_path: Optional[str]
    images: Optional[Images]
    name: Optional[str]
    original_language: Optional[str]
    original_name: Optional[str]
    overview: Optional[str]
    parts: Optional[list[Media]]
    poster_path: Optional[str]
    translations: Optional[list[Translation]]

    def __post_init__(self) -> None:
        self.name = self.name or self.original_name

    def __str__(self) -> str:
        return self.name

    @property
    def backdrop(self) -> Optional[str]:
        if self.backdrop_path:
            return f"https://image.tmdb.org/t/p/w1280{self.backdrop_path}"

    @property
    def poster(self) -> Optional[str]:
        if self.poster_path:
            return f"https://image.tmdb.org/t/p/w1280{self.poster_path}"


@dataclass
class Collections:
    page: Optional[int]
    results: Optional[list[Collection]]
    total_pages: Optional[int]
    total_results: Optional[int]

    def __bool__(self) -> bool:
        return self.total_results > 0

    def __iter__(self) -> iter:
        return iter(self.results)

    def __getitem__(self, index: int) -> Collection:
        return self.results[index]



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from typing import Optional

from .image import Images
from .alternative_name import AlternativeNames


@dataclass
class Company:
    id: Optional[int]
    alternative_names: Optional[AlternativeNames]
    description: Optional[str]
    headquarters: Optional[str]
    homepage: Optional[str]
    images: Optional[Images]
    logo_path: Optional[str]
    name: Optional[str]
    origin_country: Optional[str]
    parent_company: Optional[dict]

    def __str__(self) -> str:
        return self.name

    @property
    def logo(self) -> Optional[str]:
        if self.logo_path:
            return f"https://image.tmdb.org/t/p/w1280{self.logo_path}"


@dataclass
class Companies:
    page: Optional[int]
    results: Optional[list[Company]]
    total_pages: Optional[int]
    total_results: Optional[int]

    def __bool__(self) -> bool:
        return self.total_results > 0

    def __iter__(self) -> iter:
        return iter(self.results)

    def __getitem__(self, index: int) -> Company:
        return self.results[index]



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from datetime import date
from typing import Optional, Union


@dataclass
class Media:
    id: Optional[Union[int, float]]
    adult: Optional[bool]
    air_date: Optional[date]
    backdrop_path: Optional[str]
    character: Optional[str]
    episode_count: Optional[int]
    episode_number: Optional[int]
    first_air_date: Optional[date]
    genre_ids: Optional[list[Union[int, float]]]
    media_type: Optional[str]
    name: Optional[str]
    origin_country: Optional[list[str]]
    original_language: Optional[str]
    original_name: Optional[str]
    original_title: Optional[str]
    overview: Optional[str]
    popularity: Optional[float]
    poster_path: Optional[str]
    production_code: Optional[str]
    release_date: Optional[date]
    season_number: Optional[int]
    still_path: Optional[str]
    title: Optional[str]
    video: Optional[bool]
    vote_average: Optional[float]
    vote_count: Optional[Union[int, float]]

    def __post_init__(self) -> None:
        self.id = int(self.id)
        self.vote_count = int(self.vote_count)
        self.genre_ids = [int(genre_id) for genre_id in self.genre_ids]
        self.name = self.title or self.name or self.original_title or self.original_name
        self.original_name = self.original_title or self.original_name

    def __str__(self) -> str:
        if self.year:
            return f"{self.name} ({self.year})"
        return f"{self.name}"

    @property
    def description(self) -> Optional[str]:
        return self.overview

    @property
    def date(self) -> Optional[date]:
        return self.release_date or self.first_air_date or self.air_date

    @property
    def year(self) -> Optional[str]:
        if self.date:
            return str(self.date.year)

    @property
    def rating(self) -> Optional[str]:
        if self.vote_average and self.vote_count:
            return f"{self.vote_average} ({self.vote_count})"
        if self.vote_average:
            return str(self.vote_average)

    @property
    def backdrop(self) -> Optional[str]:
        if self.backdrop_path:
            return f"https://image.tmdb.org/t/p/w1280{self.backdrop_path}"

    @property
    def poster(self) -> Optional[str]:
        if self.poster_path:
            return f"https://image.tmdb.org/t/p/w1280{self.poster_path}"



# -*- coding: utf-8 -*-
import os
import sys
from shutil import rmtree

from setuptools import Command, find_packages, setup

with open("README.rst", "rb") as f:
    long_description = f.read().decode("utf-8")

here = os.path.abspath(os.path.dirname(__file__))


class PublishCommand(Command):
    """Support setup.py publish."""

    description = "Build and publish the package."
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print("\033[1m{0}\033[0m".format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status("Removing previous builds...")
            rmtree(os.path.join(here, "dist"))
        except Exception:
            pass

        self.status("Building Source and Wheel (universal) distribution...")
        os.system("{0} setup.py sdist bdist_wheel --universal".format(sys.executable))

        self.status("Uploading the package to PyPi via Twine...")
        os.system("twine upload dist/*")

        sys.exit()


setup(
    name="tmdb-python",
    version="0.0.3",
    description="Asynchronous Python library for The Movie Database (TMDB) API v3",
    long_description=long_description,
    url="https://github.com/leandcesar/tmdb-python",
    author="Leandro César",
    author_email="ccleandroc@gmail.com",
    license="MIT",
    packages=find_packages(exclude=("tests",)),
    keywords=[
        "movie",
        "movies",
        "tv",
        "tv show",
        "tv shows",
        "tmdb",
        "themoviedb",
        "moviedb",
        "movie database",
        "api",
        "wrapper",
    ],
    install_requires=["aiohttp"],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.9",
    ],
    cmdclass={
        "publish": PublishCommand,
    },
    zip_safe=False,
)



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from typing import Optional


@dataclass
class TranslationData:
    homepage: Optional[str]
    overview: Optional[str]
    title: Optional[str]

    def __str__(self) -> str:
        return self.title


@dataclass
class Translation:
    data: Optional[TranslationData]
    english_name: Optional[str]
    iso_3166_1: Optional[str]
    iso_639_1: Optional[str]
    name: Optional[str]

    def __str__(self) -> str:
        return self.name



# -*- coding: utf-8 -*-
from .base import Base, Response


class Keyword(Base):
    async def details(self, keyword_id: int) -> Response:
        """Get a keyword details by id.

        See more: https://developers.themoviedb.org/3/keywords/get-keyword-details
        """
        return await self.request(f"keyword/{keyword_id}")

    async def search(self, query: str, *, page: int = 1) -> Response:
        """Search for keywords.

        See more: https://developers.themoviedb.org/3/search/search-keywords
        """
        return await self.request("search/keyword", query=query, page=page)



# -*- coding: utf-8 -*-
from .base import Base, Response


class Credit(Base):
    async def details(self, credit_id: int) -> Response:
        """Get a movie or TV credit details by id.

        See more: https://developers.themoviedb.org/3/credits/get-credit-details
        """
        return await self.request(f"credit/{credit_id}")
